
#ifndef __SPMAT_H__
#define __SPMAT_H__
#include <list>
#include "spNode.h"
#include <string>  
#include <iostream>
#include <fstream>

using namespace std;

class spMat {
	// the length and the width of a matrix
	int	width, height;
	// the list containing all the elements of array.
	list< spNode < T > > ndLst;
public:
	// constructors and destructor
	spMat(int wid, int hi)
	{ width = wid; height = hi; rows = new vector< row >( height );}

	~spMat()
	{ }

	node< T > operator () ( int m, int n )
	{
		return rows[m].nodeAt(n);
	}

};

#endif


template <class T> class spMat{

private:
	list < elementNode<T> > elementList;
	int		rowNumber, colNumber;
	bool	isIndexInBound(int _rowIndex,int _colIndex);
	bool    isSameSize(int _rowNumber,int _colNumber);
	int     tempRowIndex,tempColIndex;
public :
	spMat();
	spMat(const spMat<T>& m);
	spMat(int _rowNumber, int colNumber);
	
	~spMat();

    spMat<T>& operator=(const spMat<T>& t);			
    spMat<T> operator+(const spMat<T>&  t);			
    spMat<T> operator-(const spMat<T>&  t);			
    spMat<T> operator*(const spMat<T>&  t);			
    spMat<T> operator*(T t);						
	
	proxy< T, spMat <T> > operator()(int _rowIndex, int _colIndex); 

	T		readElement();
	void	writeElement(T _value);		

	int		getRowNumber();
	int		getColNumber();

	list < elementNode<T> > getElementList();
	
	bool	loadFromFile(string _fileName);
    void	saveToFile(string _fileName);
    void    printOut();
};



template <class T>	spMat<T> operator*(T& s ,const spMat<T>& t){
	return (spMat<T>(t)*s);
}

template <class T>	const spMat<T> operator*(const spMat<T>& t,const T& s){
	return (spMat<T>(t)*s);
}

template <class T>	spMat<T> operator*(const spMat<T>& t,T& s){
	return (spMat<T>(t)*s);	
}

template <class T>	const spMat<T> operator*(const T& s,const spMat<T>& t){
	return (spMat<T>(t)*s);	
}




template <class T>	spMat<T>::spMat(){
}


template <class T>	spMat<T>::spMat(const spMat<T>& m){
	
	list < elementNode<T> >  tempList = m.elementList;

	list < elementNode<T> > ::iterator _iterator;
	rowNumber = m.rowNumber;
	colNumber = m.colNumber;
	for(_iterator=tempList.begin();_iterator!=tempList.end();++_iterator){
		elementList.push_back(*_iterator);
	}

}

template <class T>	spMat<T>::spMat(int _rowNumber,int _colNumber){
	rowNumber = _rowNumber;
	colNumber = _colNumber;
}

template <class T>	spMat<T>::~spMat(){
	elementList.clear();
}

template <class T>	bool spMat<T>::isIndexInBound(int _rowIndex,int _colIndex){
	if((_rowIndex>=rowNumber)||(_colIndex>=colNumber))
		return false;
	return true;
}

template <class T>	bool spMat<T>::isSameSize(int _rowNumber,int _colNumber){
	if (_rowNumber != rowNumber || _colNumber!=colNumber){
		cout<<"Not the same size"<<endl;
		return false;
	}
	return true;
}

template <class T>	spMat<T>& spMat<T>::operator=(const spMat<T>& t){
	if(this== &t)
		return *this;
	rowNumber = t.rowNumber;
	colNumber = t.colNumber;
	elementList = t.elementList;
	return *this;
}


template <class T>	spMat<T> spMat<T>::operator-(const spMat<T>& t){
	spMat newMatrix(*this),tempMatrix(t);
	double s = -1.;
	newMatrix = newMatrix + (tempMatrix * s);
	return newMatrix;
}
template <class T> list < elementNode<T> > spMat<T>::getElementList(){
	return elementList;
}

template <class T>  spMat<T> spMat<T>::operator*(const spMat<T>&  t){	
	spMat<T>  copyMatrix(*this),inputMatrix(t);	
	spMat<T>  newMatrix(copyMatrix.rowNumber,inputMatrix.colNumber);	

	if (copyMatrix.colNumber!=inputMatrix.rowNumber){
		cout<<" Error : the column number of first Matrix should equal to the row number of the second one!"<<endl;
		exit(1);
	}
	for(int i=0;i<copyMatrix.rowNumber;i++)
		for(int j=0;j<inputMatrix.colNumber;j++){
			T sigma = 0;
			for(int k=0;k<copyMatrix.colNumber;k++)
				sigma+= copyMatrix(i,k)*newMatrix(k,j);			
			newMatrix(i,j) = sigma;
		}
	return newMatrix;
}

template <class T>	spMat<T> spMat<T>::operator*(T t){
	spMat<T> newMatrix(*this);

	list< elementNode<T> >::iterator _iterator=newMatrix.elementList.begin();
	for(;_iterator!=newMatrix.elementList.end();++_iterator){
		int r,c;
		r = (*_iterator).rowIndex;
		c = (*_iterator).colIndex;
		newMatrix(r,c)=t*newMatrix(r,c);
	}
	return newMatrix;
}

template <class T>	spMat<T> spMat<T>::operator+(const spMat<T>& t){
	if(!isSameSize(t.rowNumber,t.colNumber))
		exit(1);

	spMat<T> newMatrix(*this),tempMatrix(t);
	
	list< elementNode<T> >::iterator _iterator=tempMatrix.elementList.begin();	

	for(; _iterator!=tempMatrix.elementList.end(); ++_iterator){
		int r,c;
		r = (*_iterator).rowIndex;	
		c = (*_iterator).colIndex;
		T temp = tempMatrix(r,c);
		T temp2 = newMatrix(r,c);
		T temp3 = temp + temp2;
		newMatrix(r,c) = temp3;
	}	
	return newMatrix;
}

template <class T>	proxy< T , spMat<T> >  spMat<T>::operator()(int _rowIndex,int _colIndex){

	void (spMat<T>::*pWriteFunction)(T) = &spMat<T>::writeElement;
	T	 (spMat<T>::*pReadFunction)()  = &spMat<T>::readElement;

	tempRowIndex = _rowIndex;
	tempColIndex = _colIndex;

	if(!isIndexInBound(_rowIndex,_colIndex)){
		cout<<"Index out of range!\n";
		proxy< T,spMat<T> >  _proxy(pWriteFunction,pReadFunction,NULL);
		return _proxy;
	}
	proxy< T,spMat<T> > _proxy(pWriteFunction,pReadFunction,this);
	return _proxy;
}

template <class T>	T spMat<T>::readElement(){
	int _rowIndex = tempRowIndex;
	int _colIndex = tempColIndex;
	list< elementNode<T> >::iterator _iterator = elementList.begin();
	for(;_iterator!=elementList.end();++_iterator){
		if((*_iterator).rowIndex==_rowIndex    &&   (*_iterator).colIndex==_colIndex)
			return (*_iterator).value;
	}
	return 0;
}

template <class T>	void spMat<T>::writeElement( T _value){
	int _rowIndex = tempRowIndex;
	int _colIndex = tempColIndex;

	list < elementNode<T> > ::iterator _iterator=elementList.begin();
	for(;_iterator!=elementList.end();++_iterator)
		if((*_iterator).rowIndex==_rowIndex    &&   (*_iterator).colIndex==_colIndex){
			(*_iterator).value = _value;
			return;
		}
	elementNode<T> newNode;
	newNode.rowIndex = _rowIndex;
	newNode.colIndex = _colIndex;
	newNode.value = _value;

	if (_value ==0)
		return;
	_iterator = elementList.begin();
	if(elementList.size()>0){
		while((_iterator !=elementList.end())&&(( *_iterator).rowIndex<_rowIndex || (*_iterator).colIndex<_colIndex))  		
			++_iterator;
	}
	//todo , should test , if change value to 0, then should erase it.
	elementList.insert(_iterator,newNode);
}

template <class T>	int spMat<T>::getRowNumber(){
	return rowNubmer;
}

template <class T>	int spMat<T>::getColNumber(){
	retrun colNumber;
}

template <class T>	bool spMat<T>::loadFromFile(string _fileName){
	ifstream	in(_fileName.c_str());
	in>>rowNumber>>colNumber;
	elementList.clear();
	if(!in)
		return false;
	while(in){
		elementNode<T>	newNode;		
		in>>newNode.rowIndex>>newNode.colIndex>>newNode.value;
		if (newNode.value==0)
			continue;
		list < elementNode<T> >::iterator _iterator=elementList.begin();
		for(;_iterator!=elementList.end();++_iterator){
			if((*_iterator).rowIndex==newNode.rowIndex  &&  (*_iterator).colIndex==newNode.colIndex){
					(*_iterator).value = newNode.value;
					break;
			}
		}
		if (_iterator==elementList.end())
			elementList.push_back(newNode);
	}	
	return true;
}

template <class T>	void spMat<T>::saveToFile(string _fileName){
	ofstream out(_fileName.c_str());
	out<<rowNumber<<"\t"<<colNumber<<"\n";

	list < elementNode<T> >::iterator _iterator=elementList.begin();
	for(;_iterator!=elementList.end();++_iterator){
		out<<"\n"<<(*_iterator).rowIndex<<"\t"<<(*_iterator).colIndex<<"\t"<<(*_iterator).value;
	}
}
template <class T> void spMat<T>::printOut(){
	cout<<"\n----------------------\n"<<endl;
	//cout<<"Here is  Matrix "<<name<<" :"<<endl;
	cout<<"Here is  Matrix  :"<<endl;
	cout<<"Row Numbers: "<<rowNumber<<"\t"<<"Column Numbers "<<colNumber<<"\n";

	list < elementNode<T> >::iterator _iterator=elementList.begin();
	for(;_iterator!=elementList.end();++_iterator){
		cout<<"( "<<(*_iterator).rowIndex<<" , "<<(*_iterator).colIndex<<" )  = "<<(*_iterator).value<<"\n";
	}
	cout<<"\n----------------------\n"<<endl;

}
#endif    
